#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const extract_dts_1 = require("../dev-tools/extract-dts");
const fs_1 = __importDefault(require("fs"));
const sade_1 = __importDefault(require("sade"));
const pkg = require('../package.json');
const prog = (0, sade_1.default)('remote-func');
prog
    .version(pkg.version);
prog
    .command('extract-dts')
    .describe('Extract .dts files and .js stub')
    .option('-s, --source', 'Source file name')
    .option('-o, --out', 'Output directory')
    .option('-tsconf, --tscConfig', 'Typescript compiler config file location')
    .example('export-dts --source my-api.js --out ./dts-api-dir')
    .action((opts) => {
    const { source, out, tscConfig } = opts;
    if (!source)
        throw new Error(`--source is mandatory`);
    if (!out)
        throw new Error(`--out is mandatory`);
    if (!fs_1.default.existsSync(source))
        throw new Error(`${source} does not exists`);
    let tsconf = {};
    if (tscConfig) {
        tsconf = JSON.parse(fs_1.default.readFileSync(tscConfig).toString());
    }
    (0, extract_dts_1.extractDts)(source, out, tsconf);
});
prog.parse(process.argv);
//# sourceMappingURL=remote-func.js.map