"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createGlobalsPlugin = void 0;
const createGlobalsPlugin = (allowedGlobals = []) => {
    const isAllowedGlobal = (name) => allowedGlobals.indexOf(name) !== -1;
    return () => {
        return {
            visitor: {
                Identifier: (path, state) => {
                    const isGlobal = state.globalsNodes.indexOf(path.node) !== -1;
                    const name = path.node.name;
                    if (isGlobal && !isAllowedGlobal(name)) {
                        throw path.buildCodeFrameError(`Unknown ${name}`);
                    }
                },
                Program: (path, state) => {
                    state.globalsNodes = Object.values(path.scope.globals || {});
                },
            },
        };
    };
};
exports.createGlobalsPlugin = createGlobalsPlugin;
//# sourceMappingURL=plugin-globals.js.map