"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createGetTrap = exports.isValidIdentifier = exports.formatBytes = exports.isProtectedProperty = exports.sizeOf = void 0;
const util_1 = require("../server/util");
// https://www.mattzeunert.com/2016/07/24/javascript-array-object-sizes.html
// Though, those sizes represents the worst case
const sizeOf = (obj) => {
    let bytes = 2;
    if (obj === null || obj === void 0) {
        bytes += 6;
    }
    else {
        switch (typeof obj) {
            case 'string':
                bytes += obj.length * 2;
                break;
            case 'number':
                bytes += 8;
                break;
            case 'boolean':
                bytes += 8;
                break;
            case 'function':
            case 'object':
                const objClass = Object.prototype.toString.call(obj).slice(8, -1);
                switch (objClass) {
                    case 'Object':
                    case 'Array':
                    case 'Function':
                        bytes += 64;
                        for (let key in obj) {
                            if (obj.hasOwnProperty(key)) {
                                bytes += (0, exports.sizeOf)(obj[key]) + (0, exports.sizeOf)(key);
                            }
                        }
                        if (objClass !== 'Function') {
                            break;
                        }
                    default:
                        bytes += (0, exports.sizeOf)(obj.toString());
                }
                break;
        }
    }
    return bytes;
};
exports.sizeOf = sizeOf;
const protectedProperty = new Map([
    'prototype',
    ...Object.getOwnPropertyNames(Object.prototype),
].map(k => [k, void 0]));
const isProtectedProperty = (prop) => protectedProperty.has(prop);
exports.isProtectedProperty = isProtectedProperty;
const formatBytes = (bytes, decimals = 2) => {
    if (bytes === 0)
        return '0 Bytes';
    const k = 1024;
    const dm = decimals < 0 ? 0 : decimals;
    const sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
    const i = Math.floor(Math.log(bytes) / Math.log(k));
    return parseFloat((bytes / Math.pow(k, i)).toFixed(dm)) + ' ' + sizes[i];
};
exports.formatBytes = formatBytes;
const identRegex = /^[$A-Z_][0-9A-Z_$]*$/i;
const isValidIdentifier = (name) => identRegex.test(name);
exports.isValidIdentifier = isValidIdentifier;
const createGetTrap = (allowedProperties) => {
    const map = new Map(allowedProperties.map(prop => [prop, true]));
    return {
        get(target, prop, receiver) {
            if (map.has(prop)) {
                return util_1.readOnlyTraps.get(target, prop, receiver);
            }
        }
    };
};
exports.createGetTrap = createGetTrap;
//# sourceMappingURL=util.js.map