import { SourceLocation } from './func';
import { RequestMessage, ResponseMessage } from './message';
export interface RequestHandlerInterface {
    getRequests(): RequestMessage[];
    write: (msg: ResponseMessage) => void;
    end: (error?: any) => void;
}
declare type Handler = (iface: RequestHandlerInterface) => void;
export interface ClientConfig {
    /** Handle requests */
    handler: Handler;
    /** Remove duplicated requests in batch mode */
    deduplicate: boolean;
    /** Execute on each func response, either result or error */
    response: (resp: ResponseMessage) => void;
}
export interface BatchConfig {
    /** Duration (in milliseconds) of batch request gathering before flush */
    timeout: number;
    /** Max number of request batched before flush */
    sizeLimit: number;
}
/** Generic client */
export declare class Client {
    private config;
    private isUsingBatch;
    private batchedRequests;
    private batchedRequestsDeferredPromises;
    private requestPromiseDedupeMap;
    private batchConfig?;
    private batchScheduleTimeout?;
    constructor(config?: Partial<ClientConfig>);
    setConfig(config: Partial<ClientConfig>): void;
    private unscheduleBatch;
    private scheduleBatch;
    /** Execute a client request */
    request(source: string, args: any[], sourceLoc?: SourceLocation): Promise<ResponseMessage>;
    /** Flush batched requests */
    flush(): void;
    /** Turn batch mode on or off */
    useBatch(config: Partial<BatchConfig> | boolean): void;
}
/** Create client instance */
export declare const createClient: (config?: Partial<ClientConfig> | undefined) => Client;
export {};
