"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.engineHandler = void 0;
/** Create an Client handler that works as a direct link between Client and Engine*/
var engineHandler = function (engine) {
    return function (iface) {
        var getRequests = iface.getRequests, write = iface.write, end = iface.end;
        try {
            var responsePromises = getRequests().map(function (request) {
                var source = request.source, args = request.args, index = request.index;
                var ctx = { source: source, args: args };
                return engine.run(source || '', args, ctx).then(function (result) {
                    write({ index: index, result: result });
                }).catch(function (err) {
                    if (err === void 0) { err = {}; }
                    var name = err.name, message = err.message;
                    write({ index: index, error: { name: name, message: message } });
                });
            });
            Promise.all(responsePromises).then(function () { return end(); }, function () { return end(); });
        }
        catch (err) {
            end(err);
            console.log(err);
        }
    };
};
exports.engineHandler = engineHandler;
//# sourceMappingURL=engine.js.map