"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.httpHandler = void 0;
var json_stream_1 = require("../json-stream");
var supportsWebStreams = (typeof global.ReadableStream !== 'undefined'
    && typeof global.TextDecoder !== 'undefined');
var handleFetchStreamResponse = function (response, write, close) {
    var _a;
    if (response.status === 207) {
        var textDecoder_1 = new TextDecoder();
        var reader_1 = (_a = response.body) === null || _a === void 0 ? void 0 : _a.getReader();
        var readChunk_1 = function (result) {
            if (result.done) {
                close();
            }
            else {
                write(textDecoder_1.decode(result.value));
                reader_1 === null || reader_1 === void 0 ? void 0 : reader_1.read().then(readChunk_1);
            }
        };
        reader_1 === null || reader_1 === void 0 ? void 0 : reader_1.read().then(readChunk_1);
    }
};
var handleFetchNoStreamResponse = function (response, write, close) {
    if (response.status === 207) {
        response.text().then(function (json) {
            write(json);
            close();
        });
    }
};
var handleFetchResponse = (supportsWebStreams
    ? handleFetchStreamResponse
    : handleFetchNoStreamResponse);
/** Create an client HTTP handler that works as a link between Client and HTTP */
var httpHandler = function (_config) {
    if (_config === void 0) { _config = {}; }
    var url = 'http://localhost/';
    var globalFetch;
    if (global && global.location) {
        url = global.location;
        globalFetch = global.fetch;
    }
    var config = __assign({ url: url, fetch: globalFetch }, _config);
    return function (iface) {
        var payload = '';
        var requests = iface.getRequests();
        var write = iface.write, end = iface.end;
        var url = config.url, fetch = config.fetch;
        var headers = {
            'Content-Type': 'text/plain;charset=utf-8',
            'Supports-Web-Streams': supportsWebStreams ? '1' : '0',
        };
        var strigifier = (0, json_stream_1.createStringifier)({
            onData: function (str) { return payload += str; }
        });
        requests.forEach(function (r) { return strigifier.write(r); });
        var parser = (0, json_stream_1.createParser)({
            onData: write,
            onClose: function () { return end(); },
            onError: function (err, str) {
                console.log("".concat(str, " ").concat(err.stack));
            },
        });
        fetch(url, { method: 'POST', headers: headers, body: payload }).then(function (response) {
            handleFetchResponse(response, parser.write, parser.close);
        }).catch(function (err) {
            // TODO: handle network error
            end(err);
            console.log(err);
        });
    };
};
exports.httpHandler = httpHandler;
//# sourceMappingURL=http.js.map