"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createStringifier = exports.createParser = void 0;
var NL_DELIMITER = '\n';
var noop = function () { };
/** create a JSON stream parser */
var createParser = function (options) {
    var _a = options.delimiter, delimiter = _a === void 0 ? NL_DELIMITER : _a, _b = options.onData, onData = _b === void 0 ? noop : _b, _c = options.onError, onError = _c === void 0 ? noop : _c, _d = options.onClose, onClose = _d === void 0 ? noop : _d;
    var remaining = '';
    var closed = false;
    var throwIfClosed = function () {
        if (closed)
            throw new Error('Parser closed');
    };
    var handleChunk = function (str) {
        try {
            var data = JSON.parse(str);
            onData(data, str);
        }
        catch (err) {
            onError(err, str);
        }
    };
    return {
        /** Write JSON chunk to parse */
        write: function (partial) {
            throwIfClosed();
            var chunks = String(partial).split(delimiter);
            var lastChunkIdx = chunks.length - 1;
            chunks.forEach(function (chunk, idx) {
                remaining = remaining + chunk;
                if (idx < lastChunkIdx) {
                    handleChunk(remaining);
                    remaining = '';
                }
            });
        },
        /** Close parser */
        close: function () {
            var chunkLen = remaining.trim().length;
            if (chunkLen > 0) {
                handleChunk(remaining);
            }
            onClose(remaining);
        }
    };
};
exports.createParser = createParser;
/** create a JSON stream stringifier */
var createStringifier = function (options) {
    var _a = options.delimiter, delimiter = _a === void 0 ? NL_DELIMITER : _a, _b = options.onData, onData = _b === void 0 ? noop : _b, _c = options.onError, onError = _c === void 0 ? noop : _c, _d = options.onClose, onClose = _d === void 0 ? noop : _d;
    var closed = false;
    var throwIfClosed = function () {
        if (closed)
            throw new Error('Stringifier closed');
    };
    return {
        /** Write object to stringify */
        write: function (data) {
            throwIfClosed();
            try {
                var str = JSON.stringify(data) + delimiter;
                onData(str, data);
            }
            catch (err) {
                onError(err, data);
            }
        },
        /** Close stringifier */
        close: function () {
            onClose();
        }
    };
};
exports.createStringifier = createStringifier;
//# sourceMappingURL=json-stream.js.map