export interface CacheConfig {
    /** Max number of entries to keep in the cache */
    maxEntriesCount: number;
    /** Max time an entry can be stored in the cache without being retrieved */
    maxIddleTime: number;
    /** Interval (in milliseconds) to run garbage collector*/
    gcIntervalTime: number;
}
export interface CacheEntry<T> {
    data: T;
    createdAt: number;
    usedAt: number;
}
/** Cache with garbage collector */
export declare class Cache<T> {
    private map;
    private config;
    constructor(config?: Partial<CacheConfig>);
    /** Run garbage collector */
    private runGc;
    /** Retrieve cache entry */
    get(key: string): T | void;
    /** Store cache entry */
    set(key: string, data: T): Map<string, CacheEntry<T>>;
    /** Delete cache entry */
    delete(key: string): boolean;
    /** Return size of the cache */
    size(): number;
}
