"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cache = void 0;
const util_1 = require("./util");
/** Cache with garbage collector */
class Cache {
    constructor(config = {}) {
        /** Run garbage collector */
        this.runGc = () => {
            const { maxEntriesCount, maxIddleTime } = this.config;
            const size = this.map.size;
            const garbageKeys = [];
            const now = Date.now();
            const idxEndRemoval = size - maxEntriesCount;
            let count = 0;
            this.map.forEach((entry, key) => {
                if (count <= idxEndRemoval) {
                    // extra
                    garbageKeys.push(key);
                }
                if (now - entry.usedAt >= maxIddleTime) {
                    // iddle
                    garbageKeys.push(key);
                }
                count++;
            });
            // remove garbage
            garbageKeys.forEach(key => this.delete(key));
        };
        this.map = new Map();
        this.config = Object.assign({ maxEntriesCount: 10000, maxIddleTime: (0, util_1.mins)(10), gcIntervalTime: (0, util_1.secs)(10) }, config);
        // start gc
        setInterval(this.runGc, this.config.gcIntervalTime);
    }
    /** Retrieve cache entry */
    get(key) {
        const entry = this.map.get(key);
        if (entry) {
            entry.usedAt = Date.now();
            return entry.data;
        }
    }
    /** Store cache entry */
    set(key, data) {
        const now = Date.now();
        return this.map.set(key, {
            data,
            createdAt: now,
            usedAt: now
        });
    }
    /** Delete cache entry */
    delete(key) {
        return this.map.delete(key);
    }
    /** Return size of the cache */
    size() {
        return this.map.size;
    }
}
exports.Cache = Cache;
//# sourceMappingURL=cache.js.map