import { ServiceBaseContext, Services } from './service';
import { RequestContext } from './http';
export interface EngineConfig {
    /** Dictionary of services */
    services: Services;
    /** Path to a file containing services */
    servicesPath: String;
    /** Transform query context in service context */
    context: (reqCtx: RequestContext) => ServiceBaseContext;
    /** Read source dynamically */
    source: (src: string) => Promise<string> | string;
    /** Max execution time for query functions */
    timeout: number;
    /** Whether display query errors or not */
    displayErrors: boolean;
}
/** Run JavaScript query functions */
export declare class Engine {
    private config;
    private cfuncCache;
    private services;
    private servicesKeys;
    private servicesModule;
    private readSource;
    constructor(config: Partial<EngineConfig>);
    /** Return engine configuration */
    getConfig(): Partial<EngineConfig>;
    /** Run query function */
    run(source: string, args?: any[], queryContext?: any): Promise<any>;
    private execute;
    private parseFunc;
    private handleRpc;
    /** Return path of registered endpoints */
    getEndpointPaths(): string[];
}
/** Create new Engine */
export declare const createEngine: (config?: Partial<EngineConfig>) => Engine;
