"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.expressHandler = void 0;
const http_1 = require("../http");
/** Create an Express middleware that works as a link between HTTP and Engine*/
const expressHandler = (engine) => {
    const express = require('express');
    const text = express.text();
    return (request, response, next) => {
        text(request, response, async () => {
            await (0, http_1.handleHttpRequest)({
                initialContext: { request, response },
                engine: engine,
                getQueryParam(key) {
                    return request.query[key];
                },
                getMethod() {
                    return request.method;
                },
                getBody() {
                    return request.body;
                },
                getHeader(name) {
                    return request.get(name);
                },
                setHeader(name, value) {
                    return response.set(name, value);
                },
                setStatusCode(statusCode) {
                    response.statusCode = statusCode;
                },
                write(data) {
                    if (response.finished || response.writableEnded) {
                        throw new Error('Response ended');
                    }
                    response.write(data);
                },
                end() {
                    response.end();
                }
            });
            return next();
        });
    };
};
exports.expressHandler = expressHandler;
//# sourceMappingURL=express.js.map