"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.microHandler = void 0;
const http_1 = require("../http");
/** Create a Micro handler that works as a link between HTTP and Engine */
const microHandler = (engine) => {
    const { text } = require('micro');
    return async (request, response) => {
        const url = new URL(request.url || '', `http://${request.headers.host}`);
        const body = await text(request);
        await (0, http_1.handleHttpRequest)({
            initialContext: { request, response },
            engine: engine,
            getQueryParam(key) {
                return url.searchParams.get(key);
            },
            getMethod() {
                return request.method;
            },
            getBody() {
                return body;
            },
            getHeader(name) {
                return String(request.headers[name]);
            },
            setHeader(name, value) {
                return response.setHeader(name, value);
            },
            setStatusCode(statusCode) {
                response.statusCode = statusCode;
            },
            write(data) {
                if (response.finished || response.writableEnded) {
                    throw new Error('Response ended');
                }
                response.write(data);
            },
            end() {
                response.end();
            }
        });
    };
};
exports.microHandler = microHandler;
//# sourceMappingURL=micro.js.map