"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.nextHandler = void 0;
const http_1 = require("../http");
/** Create an Express handler that works as a link between HTTP and Engine*/
const nextHandler = (engine) => {
    return async (request, response) => {
        await (0, http_1.handleHttpRequest)({
            initialContext: { request, response },
            engine: engine,
            getQueryParam(key) {
                return request.query[key];
            },
            getMethod() {
                return request.method;
            },
            getBody() {
                return request.body;
            },
            getHeader(name) {
                return String(request.headers[name]);
            },
            setHeader(name, value) {
                return response.setHeader(name, value);
            },
            setStatusCode(statusCode) {
                response.statusCode = statusCode;
            },
            write(data) {
                if (response.finished || response.writableEnded) {
                    throw new Error('Response ended');
                }
                response.write(data);
            },
            end() {
                response.end();
            }
        });
    };
};
exports.nextHandler = nextHandler;
//# sourceMappingURL=next.js.map