"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.instantiateServices = exports.createService = exports.AsyncResult = void 0;
const util_1 = require("./util");
const SERVICE = Symbol('service');
const SERVICE_NAME = Symbol('service-name');
// TypeScript issue: https://github.com/microsoft/TypeScript/issues/12776
exports.AsyncResult = Promise;
const isService = (service) => {
    return service && service[SERVICE];
};
const getContextLogger = (ctx) => {
    return (ctx && (0, util_1.isFunction)(ctx === null || ctx === void 0 ? void 0 : ctx.log) ? ctx === null || ctx === void 0 ? void 0 : ctx.log : util_1.noop);
};
const instantiateService = (service, ctx) => {
    let log = getContextLogger(ctx);
    const serviceInstance = service(ctx);
    return (0, util_1.readOnly)(serviceInstance, {
        get(target, prop) {
            const endpoint = target[prop];
            if (typeof endpoint !== 'function') {
                throw new Error(`'${prop}' is not a function`);
            }
            return (...args) => {
                log({
                    type: 'call-endpoint',
                    service: service[SERVICE_NAME],
                    endpoint: prop,
                    args
                });
                return (0, util_1.deepClone)(endpoint.apply(serviceInstance, args));
            };
        }
    });
};
const readServices = (services) => {
    if (!services) {
        throw new Error('invalid services');
    }
    if (services.default !== void 0) {
        throw new Error('services should not export default');
    }
    Object.entries(services).forEach(([key, service]) => {
        if (!isService(service)) {
            throw new Error(`invalid service: '${key}'`);
        }
    });
    return Object.assign({}, services);
};
/** Create a service */
const createService = (serviceFactory) => {
    serviceFactory[SERVICE] = true;
    return serviceFactory;
};
exports.createService = createService;
/** Instantiate and bind services to a provided context */
const instantiateServices = (services, ctx) => {
    const _services = readServices(services);
    const servicesMemo = new Map();
    return (0, util_1.readOnly)(_services, {
        get(target, prop) {
            let serviceInstance = servicesMemo.get(prop);
            if (!serviceInstance) {
                const service = target[prop];
                if (isService(service)) {
                    service[SERVICE_NAME] = prop;
                }
                else {
                    return service;
                }
                serviceInstance = instantiateService(service, ctx);
                servicesMemo.set(prop, serviceInstance);
            }
            return serviceInstance;
        },
    });
};
exports.instantiateServices = instantiateServices;
//# sourceMappingURL=service.js.map