"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deepClone = exports.readOnly = exports.readOnlyTraps = exports.getConsole = exports.isProduction = exports.isString = exports.isArray = exports.isThenable = exports.isPrimitive = exports.isFunction = exports.isObject = exports.mins = exports.secs = exports.noop = exports.identity = exports.isReadOnly = void 0;
const readOnlySymbol = Symbol('read-only');
const isReadOnly = (obj) => !(0, exports.isPrimitive)(obj) && obj[readOnlySymbol];
exports.isReadOnly = isReadOnly;
const identity = (a) => a;
exports.identity = identity;
const noop = () => { };
exports.noop = noop;
const secs = (s) => s * 1000;
exports.secs = secs;
const mins = (m) => (0, exports.secs)(m) * 60;
exports.mins = mins;
const isObject = (obj) => typeof obj === 'object' && obj !== null;
exports.isObject = isObject;
const isFunction = (fn) => typeof fn === 'function';
exports.isFunction = isFunction;
const isPrimitive = (v) => v == null || (!(0, exports.isFunction)(v) && !(0, exports.isObject)(v));
exports.isPrimitive = isPrimitive;
const isThenable = (v) => v && (0, exports.isFunction)(v.then);
exports.isThenable = isThenable;
exports.isArray = Array.isArray.bind(Array);
const isString = (v) => typeof v === 'string';
exports.isString = isString;
const isProduction = () => {
    const { NODE_ENV } = process.env;
    return NODE_ENV === 'production';
};
exports.isProduction = isProduction;
const getConsole = () => ((0, exports.isProduction)()
    ? { log: exports.noop, warn: exports.noop, error: exports.noop }
    : console);
exports.getConsole = getConsole;
const hasOwnProperty = Object.prototype.hasOwnProperty;
exports.readOnlyTraps = {
    construct(target, args) {
        return (0, exports.readOnly)(new target(...args));
    },
    get(target, prop, receiver) {
        const val = target[prop];
        if ((0, exports.isFunction)(val)) {
            return (0, exports.readOnly)((...args) => val.apply(target, args));
        }
        return (0, exports.readOnly)(Reflect.get(target, prop, receiver));
    },
    set(target, prop, val) {
        return val;
    },
};
const readOnly = (target, traps = {}) => {
    if ((0, exports.isPrimitive)(target) || (0, exports.isReadOnly)(target)) {
        return target;
    }
    else {
        target[readOnlySymbol] = true;
        return new Proxy(target, Object.assign(Object.assign({}, exports.readOnlyTraps), traps));
    }
};
exports.readOnly = readOnly;
const MAX_DEPTH = 100;
const deepClone = (o, maxDepth = MAX_DEPTH) => {
    if (maxDepth < 0)
        return void 0;
    // if not array or object or is null return self
    if ((0, exports.isFunction)(o))
        return void 0;
    if (!(0, exports.isObject)(o))
        return o;
    // date
    if (o instanceof Date) {
        return new Date(o.getTime());
    }
    // toJSON
    if ((0, exports.isFunction)(o.toJSON)) {
        o = o.toJSON();
    }
    // promise
    if ((0, exports.isThenable)(o)) {
        const nextMaxDepth = maxDepth - 1;
        return o.then((o) => (0, exports.deepClone)(o, nextMaxDepth));
    }
    // array
    if ((0, exports.isArray)(o)) {
        let len = o.length;
        let newO = [];
        const nextMaxDepth = maxDepth - 1;
        for (let i = 0; i < len; i++) {
            newO[i] = (0, exports.deepClone)(o[i], nextMaxDepth);
        }
        return newO;
    }
    // object
    let newO = {};
    for (let propName in o) {
        const nextMaxDepth = maxDepth - 1;
        if (hasOwnProperty.call(o, propName)) {
            newO[propName] = (0, exports.deepClone)(o[propName], nextMaxDepth);
        }
    }
    return newO;
};
exports.deepClone = deepClone;
//# sourceMappingURL=util.js.map